module Instruction where

data Instruction = 
    Operand0 String |
    OperandInt1 String Int |
    OperandInt2 String Int Int |
    OperandString1 String String |
    FlagResetDontCare |
    FlagResetRegistersSafe |
    Label String deriving Eq

instance Show Instruction where
     show (Operand0 ins) = ins
     show (OperandInt1 ins op1) = ins ++ " " ++ hexNumber op1
     show (OperandString1 ins op1) = ins ++ " " ++ op1
     show (Label l) = l ++ ":"
     show FlagResetDontCare = "[FlagResetDontCare]"
     show FlagResetRegistersSafe = "[FlagResetRegistersSafe]"

hexNumber i | i < 16 = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"] !! i
hexNumber i = error ("too large integer " ++ show i)

isInstruction (Label _) = False
isInstruction _ = True

isLabel (Label _) = True
isLabel _ = False

isBreakRegisterA (Operand0 "KA") = True
isBreakRegisterA (Operand0 "CH") = True
isBreakRegisterA (Operand0 "CY") = True
isBreakRegisterA (Operand0 "MA") = True
isBreakRegisterA (Operand0 "M+") = True
isBreakRegisterA (Operand0 "M-") = True
isBreakRegisterA (OperandInt1 "TIA" _) = True
isBreakRegisterA (OperandInt1 "AIA" _) = True
isBreakRegisterA (OperandString1 "CAL" _) = True -- FIXME
isBreakRegisterA _ = False

isBreakRegisterY (Operand0 "CH") = True
isBreakRegisterY (Operand0 "CY") = True
isBreakRegisterY (OperandInt1 "TIY" _) = True
isBreakRegisterY (OperandInt1 "AIY" _) = True
isBreakRegisterY (OperandString1 "CAL" _) = True -- FIXME
isBreakRegisterY _ = False

isResetFlag (Operand0 "KA") = False
isResetFlag (Operand0 "M+") = False
isResetFlag (Operand0 "M-") = False
isResetFlag (OperandInt1 "AIA" _) = False
isResetFlag (OperandInt1 "AIY" _) = False
isResetFlag (OperandInt1 "CIA" _) = False
isResetFlag (OperandInt1 "CIY" _) = False
isResetFlag (OperandString1 "CAL" "SIFT") = False
isResetFlag (Label _) = False
isResetFlag _ = True
