module Lexer where

import Char

data Token =
    TokenIdentifier String |
    TokenSymbol String |
    TokenNumber Int
    deriving (Show, Eq)

lexer = removeComment 0 . firstLex

removeComment n (TokenSymbol "/*":tokens) = removeComment (n + 1) tokens
removeComment n (TokenSymbol "*/":tokens) = removeComment (n - 1) tokens
removeComment n (token:tokens) | n > 0 = removeComment n tokens
removeComment n (token:tokens) = token : removeComment n tokens
removeComment _ [] = []

firstLex [] = []
firstLex s =
    if fst number /= "" then
        TokenNumber (read (fst number)) : firstLex (snd number)
    else if fst alphas /= "" then
             TokenIdentifier (fst alphas) : firstLex (snd alphas)
         else if fst symbols /= "" then
                  TokenSymbol (fst symbols) : firstLex (snd symbols)
              else if fst symbol /= "" then
                        TokenSymbol (fst symbol) : firstLex (snd symbol)
                    else
                        firstLex (snd alphas)
    where
      alphas = tokenerWord isAlpha isAlphaNum s ""
      number = tokenerWord isDigit isAlphaNum s ""
      symbol = tokenerWord fp sp s "" where
          fp x = ((not.isAlphaNum) x) && ((not.isSpace) x)
          sp x = False
      symbols = tokenerWord fp fp s "" where
          fp x = ((not.isAlphaNum) x) && ((not.isSpace) x) &&
                 not (any (x==) ['{', '}', '(', ')', ';'])

tokenerWord fp sp (c:cs) "" | fp c = tokenerWord fp sp cs [c]
tokenerWord fp sp (c:cs) "" = ("", cs)
tokenerWord fp sp (c:cs) s | sp c = tokenerWord fp sp cs (s ++ [c])
tokenerWord fp sp cs s = (s, cs)
